﻿using HarmonyLib;
using System.Reflection;
using UnityEngine;
using RaftModLoader;
using HMLLibrary;
using System.Collections.Generic;
using System.Runtime.CompilerServices;
using UnityEngine.UI;
using I2.Loc;

namespace BulkCrafting
{
    public class Main : Mod
    {
        public static readonly string ModNamePrefix = "<color=#d16e17>Bulk Crafting</color>";

        string bulkKey;
        int bulkAmount;
        bool ExtraSettingsAPI_Loaded;
        public bool KeyDown => ExtraSettingsAPI_Loaded ? MyInput.GetButton(bulkKey) : (Input.GetKey(KeyCode.LeftShift) || Input.GetKey(KeyCode.RightShift));
        public int AmountMultiplier => ExtraSettingsAPI_Loaded ? bulkAmount : DefaultMultiplier;
        bool wasPressed;
        public const int DefaultMultiplier = 10;


        Harmony harmony;

        Item_Base lastItem;
        Dictionary<CostMultiple, int> changedCosts = new Dictionary<CostMultiple, int>();
        int amountToCraft;

        public void Start()
        {
            (harmony = new Harmony("com.thmsn.bulk-crafting")).PatchAll(Assembly.GetExecutingAssembly());
            Debug.Log(ModNamePrefix + " has been loaded!");
        }

        public void OnModUnload()
        {
            harmony.UnpatchAll(harmony.Id);
            UpdateSelectedItem(null);
            Debug.Log(ModNamePrefix + " has been unloaded!");
        }

        void FixedUpdate() => Update();
        void Update()
        {
            var selectedRecipe = ComponentManager<CraftingMenu>.Value?.selectedRecipeBox?.ItemToCraft;
            UpdateSelectedItem(selectedRecipe ? selectedRecipe : null);
        }

        public void UpdateSelectedItem(Item_Base item)
        {
            var update = false;
            if (item != lastItem)
            {
                if (lastItem)
                {
                    lastItem.settings_recipe.SetAmountToCraft(amountToCraft);
                    foreach (var p in changedCosts)
                        p.Key.amount = p.Value;
                    changedCosts.Clear();
                }
                lastItem = item;
                if (item)
                {
                    amountToCraft = item.settings_recipe.AmountToCraft;
                    foreach (var c in item.settings_recipe.NewCost)
                        changedCosts[c] = c.amount;
                    update = true;
                }
            }
            var down = KeyDown;
            if (wasPressed != down || update)
            {
                wasPressed = down;
                var a = down ? AmountMultiplier : 1;
                foreach (var p in changedCosts)
                    p.Key.amount = p.Value * a;
                ComponentManager<CraftingMenu>.Value.selectedRecipeBox.craftButton.GetComponentInChildren<Localize>()?.OnLocalize(true);
                if (lastItem)
                {
                    lastItem.settings_recipe.SetAmountToCraft(amountToCraft * a);
                    ComponentManager<CraftingMenu>.Value.selectedRecipeBox.UpdateCraftAmountText(lastItem);
                    if (down)
                        ComponentManager<CraftingMenu>.Value.selectedRecipeBox.craftButton.GetComponentInChildren<Text>().text += " x" + a;
                }
            }
        }

        public void ExtraSettingsAPI_Load()
        {
            bulkKey = ExtraSettingsAPI_GetKeybindName(nameof(bulkKey));
            ExtraSettingsAPI_SettingsClose();
            ExtraSettingsAPI_Unload();
        }
        public void ExtraSettingsAPI_Unload()
        {
            if (bulkAmount != DefaultMultiplier)
                wasPressed = false;
        }
        public void ExtraSettingsAPI_SettingsClose()
        {
            if (!int.TryParse(ExtraSettingsAPI_GetInputValue(nameof(bulkAmount)), out var newAmount))
                newAmount = DefaultMultiplier;
            if (bulkAmount == newAmount)
                return;
            wasPressed = false;
            bulkAmount = newAmount;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static string ExtraSettingsAPI_GetKeybindName(string SettingName) => "";

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static string ExtraSettingsAPI_GetInputValue(string SettingName) => "";
    }

    public static class ExtentionMethods
    {
        static FieldInfo _AmountToCraft = AccessTools.Field(typeof(ItemInstance_Recipe), "amountToCraft");
        public static void SetAmountToCraft(this ItemInstance_Recipe recipe, int newAmount) => _AmountToCraft.SetValue(recipe, newAmount);
        static MethodInfo _UpdateCraftAmountText = AccessTools.Method(typeof(SelectedRecipeBox), nameof(UpdateCraftAmountText));
        public static void UpdateCraftAmountText(this SelectedRecipeBox recipeBox, Item_Base item) => _UpdateCraftAmountText.Invoke(recipeBox, new object[] { item });
    }
}