﻿using HarmonyLib;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Reflection.Emit;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;

/// <summary>
/// Inventory.AddItem(string,int) is being patched to make it so if an item amount is being added that is greater than the stack size, the itme is added recursively instead of all going into one slot
/// </summary>
[HarmonyPatch(typeof(Inventory), "AddItem", typeof(string), typeof(int))]
static class Fix_Inventory_AddItem
{
    static IEnumerable<CodeInstruction> Transpiler(IEnumerable<CodeInstruction> instructions)
    {
        var code = instructions.ToList();
        code.InsertRange(
            code.FindIndex(x => x.opcode == OpCodes.Callvirt && x.operand is MethodInfo m && m.Name == "get_IsEmpty") + 1,
            new[] {
                new CodeInstruction(OpCodes.Ldloc_0),
                new CodeInstruction(OpCodes.Ldarg_2),
                new CodeInstruction(OpCodes.Call,AccessTools.Method(typeof(Fix_Inventory_AddItem),nameof(ShouldSetSlot)))
            });
        return code;
    }

    static bool ShouldSetSlot(bool original, Item_Base item, int amount) => original && item.settings_Inventory.StackSize >= amount;
}